@@ -12,10 +12,11 @@ __credits__ = [
     "Andy Denmark, https://github.com/denmark",
 ]
 __license__ = "GPL 3"
-__version__ = "1.4.1"
+__version__ = "1.4.2"
 __email__ = "versae@gmail.com"
 __url__ = "https://github.com/versae/neo4j-rest-client"
-__description__ = """Library to interact with Neo4j standalone REST server"""
+__description__ = """Object-oriented Python library to interact with """ \
+                  """Neo4j standalone REST server"""
 __status__ = "Development"
 # Order
 BREADTH_FIRST = "breadth_first"
