@@ -110,6 +110,19 @@ class NodesTestCase(unittest.TestCase):
         n2 = self.gdb.node[n1.id]
         self.assertEqual(n1.properties, n2.properties)
 
+    def test_set_node_property_safe(self):
+        n1 = self.gdb.node(language="Español antigüillo")
+        n1.set("Idioma de los subtítulos", "Español antigüillo")
+        self.assertEqual(n1.get("Idioma de los subtítulos"), n1.get("language"))
+
+    def test_set_node_properties_safe(self):
+        n1 = self.gdb.node()
+        n1.properties = {"Idioma de los subtítulos": "Español antigüillo"}
+        n1_properties = n1.properties
+        n2 = self.gdb.node[n1.id]
+        n2_properties = n2.properties
+        self.assertEqual(n1_properties, n2_properties)
+
     def test_delete_node_property_dictionary(self):
         n1 = self.gdb.node(name="John Doe", profession="Hacker")
         del n1["name"]
@@ -150,6 +163,15 @@ class RelationshipsTestCase(NodesTestCase):
         rel = n1.relationships.create("Knows", n2)
         self.assertEqual(rel.properties, {})
 
+    def test_create_relationship_functional_safe(self):
+        n1 = self.gdb.nodes.create()
+        n2 = self.gdb.nodes.create()
+        r1 = n1.relationships.create("recommends ñapa motörhead", n2)
+        self.assertEqual(r1.properties, {})
+        self.assertEqual(r1.type, "recommends ñapa motörhead")
+        r2 = self.gdb.relationships.get(r1.id)
+        self.assertEqual(r1.type, r2.type)
+
     def test_create_relationship_properties(self):
         n1 = self.gdb.node()
         n2 = self.gdb.node()
@@ -162,6 +184,25 @@ class RelationshipsTestCase(NodesTestCase):
         rel = n1.relationships.create("Knows", n2, since=1970)
         self.assertEqual(rel.properties, {"since": 1970})
 
+    def test_set_relationship_property_safe(self):
+        n1 = self.gdb.nodes.create()
+        n2 = self.gdb.nodes.create()
+        rel = n1.relationships.create("Knows", n2,
+                                      language="Español antigüillo")
+        rel.set("Idioma de los subtítulos", "Español antigüillo")
+        self.assertEqual(rel.get("Idioma de los subtítulos"),
+                         rel.get("language"))
+
+    def test_set_relationship_properties_safe(self):
+        n1 = self.gdb.nodes.create()
+        n2 = self.gdb.nodes.create()
+        r1 = n1.relationships.create("Knows", n2)
+        r1.properties = {"Idioma de los subtítulos": "Español antigüillo"}
+        r1_properties = r1.properties
+        r2 = self.gdb.relationships.get(r1.id)
+        r2_properties = r2.properties
+        self.assertEqual(r1_properties, r2_properties)
+
     def test_delete_relationship(self):
         n1 = self.gdb.nodes.create()
         n2 = self.gdb.nodes.create()
@@ -173,8 +214,16 @@ class RelationshipsTestCase(NodesTestCase):
         n1 = self.gdb.nodes.create()
         n2 = self.gdb.nodes.create()
         rel = n1.relationships.create("Knows", n2, since=1970)
-        r = self.gdb.relationships.get(0)
-        self.assertIsInstance(r, client.Relationship)
+        self.assertIsInstance(rel, client.Relationship)
+
+    def test_delete_relationship_not_found(self):
+        n1 = self.gdb.nodes.create()
+        n2 = self.gdb.nodes.create()
+        rel = n1.relationships.create("Knows", n2, since=1970)
+        rel_id = rel.id
+        rel.delete()
+        self.assertRaises(request.NotFoundError, self.gdb.relationships.get,
+                          rel_id)
 
 
 class IndexesTestCase(RelationshipsTestCase):
@@ -197,6 +246,14 @@ class IndexesTestCase(RelationshipsTestCase):
         index["bands"]["AC/DC"] = n1
         self.assertTrue(n1 in index["bands"]["AC/DC"])
 
+    def test_delete_index_for_nodes(self):
+        n1 = self.gdb.nodes.create(name="John Doe", place="Texas")
+        index = self.gdb.nodes.indexes.create(name="doe")
+        index["surnames"]["d"] = n1
+        index.delete()
+        self.assertRaises(request.NotFoundError,
+                          index["surnames"].__getitem__, "d")
+
     def test_create_index_for_relationships(self):
         n1 = self.gdb.nodes.create(name="John Doe", place="Texas")
         n2 = self.gdb.nodes.create(name="Michael Doe", place="Tijuana")
@@ -221,6 +278,16 @@ class IndexesTestCase(RelationshipsTestCase):
         index.delete("feeling", "hate", r1)
         self.assertTrue(r1 not in index["feeling"]["hate"])
 
+    def test_delete_index_for_relationships(self):
+        n1 = self.gdb.nodes.create(name="John Doe", place="Texas")
+        n2 = self.gdb.nodes.create(name="Michael Doe", place="Tijuana")
+        r1 = self.gdb.relationships.create(n1, "Hates", n2)
+        index = self.gdb.relationships.indexes.create(name="brothers")
+        index["feeling"]["hate"] = r1
+        index.delete()
+        self.assertRaises(request.NotFoundError,
+                          index["feeling"].__getitem__, "hate")
+
     def test_query_index(self):
         n1 = self.gdb.nodes.create(name="John Doe", place="Texas")
         n2 = self.gdb.nodes.create(name="Michael Donald", place="Tijuana")
